@@ -14,7 +14,9 @@ from tests.fixtures.im_fixtures import (
     im_gry_np,
     im_mch_np,
     im_rgb_np,
+    im_rgb_np_uneven,
     mask_np,
+    mask_geojson,
     zarr_im_gry_np,
     zarr_im_mch_np,
     zarr_im_rgb_np,
