@@ -1,8 +1,6 @@
-from .reg_image import RegImage  # noqa: F401
-from .tifffile_reg_image import TiffFileRegImage  # noqa: F401
-from .czi_reg_image import CziRegImage  # noqa: F401
-from .ometiff_reg_image import OmeTiffRegImage  # noqa: F401
-from .sitk_reg_image import SitkRegImage  # noqa: F401
-from .aics_reg_image import AICSRegImage  # noqa: F401
 from .np_reg_image import NumpyRegImage  # noqa: F401
+from .sitk_reg_image import SitkRegImage  # noqa: F401
+from .tifffile_reg_image import TiffFileRegImage  # noqa: F401
+from .aics_reg_image import AICSRegImage  # noqa: F401
+from .czi_reg_image import CziRegImage  # noqa: F401
 from .merge_reg_image import MergeRegImage  # noqa: F401
