@@ -8,7 +8,7 @@ import geojson
 import numpy as np
 import SimpleITK as sitk
 
-from wsireg.reg_transform import RegTransform
+from wsireg.reg_transforms.reg_transform import RegTransform
 from wsireg.utils.tform_utils import wsireg_transforms_to_itk_composite
 
 GJ_SHAPE_TYPE = {
@@ -19,6 +19,14 @@ GJ_SHAPE_TYPE = {
     "multilinestring": geojson.MultiLineString,
     "linestring": geojson.LineString,
 }
+GJ_SHAPE_TYPE_NAME = {
+    "polygon": "Polygon",
+    "multipolygon": "MultiPolygon",
+    "point": "Point",
+    "multipoint": "MultiPoint",
+    "multilinestring": "MultiLineString",
+    "linestring": "LineString",
+}
 
 
 def gj_to_np(gj: dict):
@@ -60,11 +68,18 @@ def gj_to_np(gj: dict):
     else:
         shape_name = gj.get("properties").get("classification").get("name")
 
-    return {
-        "array": pts.astype(np.double),
-        "shape_type": gj.get("geometry").get("type"),
-        "shape_name": shape_name,
-    }
+    if len(pts.shape) == 1:
+        return {
+            "array": np.asarray(pts[0]).astype(np.double),
+            "shape_type": gj.get("geometry").get("type"),
+            "shape_name": shape_name,
+        }
+    else:
+        return {
+            "array": pts.astype(np.double),
+            "shape_type": gj.get("geometry").get("type"),
+            "shape_name": shape_name,
+        }
 
 
 def add_unamed(gj):
@@ -99,6 +114,9 @@ def read_geojson(json_file: str):
                     data = f.read()
                     gj_data = json.loads(data.decode("utf-8"))
 
+    if isinstance(gj_data, dict):
+        gj_data = [gj_data]
+
     shapes_np = [gj_to_np(s) for s in gj_data]
     gj_data = [add_unamed(gj) for gj in gj_data]
     return gj_data, shapes_np
@@ -130,11 +148,24 @@ def np_to_geojson(
     sh_type = shape_type.lower()
 
     gj_func = GJ_SHAPE_TYPE[sh_type]
+    if sh_type == "polygon":
+        np_array = np.vstack([np_array, np_array[0, :]])
+        geometry = gj_func([np_array.tolist()])
+    elif sh_type in ["multipoint", "linestring"]:
+        geometry = gj_func(np_array.transpose().tolist())
+    else:
+        geometry = gj_func(np_array.tolist())
 
     shape_gj = {
-        "geometry": gj_func(np_array.tolist()),
-        "properties": {"classification": {"name": shape_name}},
+        "type": "Feature",
+        "id": "annotation",
+        "geometry": geometry,
+        "properties": {
+            "classification": {"name": shape_name, "colorRGB": -1},
+            "isLocked": False,
+        },
     }
+
     shape_np = {
         "array": np_array,
         "shape_type": shape_type.lower(),
