@@ -10,7 +10,7 @@ from wsireg.parameter_maps.transformations import (
     BASE_AFF_TFORM,
     BASE_RIG_TFORM,
 )
-from wsireg.reg_transform import RegTransform
+from wsireg.reg_transforms.reg_transform import RegTransform
 from wsireg.utils.itk_im_conversions import (
     itk_image_to_sitk_image,
     sitk_image_to_itk_image,
@@ -587,181 +587,3 @@ def identity_elx_transform(
     identity.update({"Size": [str(i) for i in image_size]})
     identity.update({"Spacing": [str(i) for i in image_spacing]})
     return identity
-
-
-#
-# def apply_transform_dict(
-#     image_fp,
-#     image_res,
-#     tform_dict_in,
-#     prepro_dict=None,
-#     is_shape_mask=False,
-#     writer="sitk",
-#     **im_tform_kwargs,
-# ):
-#     """
-#     Apply a complex series of transformations in a python dictionary to an image
-#
-#     Parameters
-#     ----------
-#     image_fp : str
-#         file path to the image to be transformed, it will be read in it's entirety
-#     image_res : float
-#         pixel resolution of image to be transformed
-#     tform_dict : dict of lists
-#         dict of SimpleElastix transformations stored in lists, may contain an "initial" transforms (preprocessing transforms)
-#         these will be applied first, then the key order of the dict will determine the rest of the transformations
-#     prepro_dict : dict
-#         preprocessing to perform on image before transformation, default None reads full image
-#     is_shape_mask : bool
-#         whether the image being transformed is a shape mask (determines import)
-#
-#     Returns
-#     -------
-#     image: itk.Image
-#         image that has been transformed
-#
-#     """
-#
-#     if is_shape_mask is False:
-#         if isinstance(image_fp, sitk.Image):
-#             image = image_fp
-#         # else:
-#         #     image = RegImage(
-#         #         image_fp, image_res, prepro_dict=prepro_dict
-#         #     ).image
-#     else:
-#         image = sitk.GetImageFromArray(image_fp)
-#         del image_fp
-#         image.SetSpacing((image_res, image_res))
-#
-#     if tform_dict_in is None:
-#         if writer == "zarr":
-#             image = transform_2d_image(
-#                 image,
-#                 None,
-#                 writer="zarr",
-#                 zarr_store_dir=im_tform_kwargs["zarr_store_dir"],
-#                 channel_names=im_tform_kwargs["channel_names"],
-#                 channel_colors=im_tform_kwargs["channel_colors"],
-#             )
-#         else:
-#             image = transform_2d_image(image, None)
-#
-#     else:
-#         tform_dict = tform_dict_in.copy()
-#
-#         if tform_dict.get("registered") is None and tform_dict.get(0) is None:
-#             tform_dict["registered"] = tform_dict["initial"]
-#             tform_dict.pop("initial", None)
-#
-#             if isinstance(tform_dict.get("registered"), list) is False:
-#                 tform_dict["registered"] = [tform_dict["registered"]]
-#
-#             for idx in range(len(tform_dict["registered"])):
-#                 tform_dict[idx] = [tform_dict["registered"][idx]]
-#
-#             tform_dict.pop("registered", None)
-#         else:
-#             tform_dict = prepare_tform_dict(tform_dict, shape_tform=False)
-#
-#         if "initial" in tform_dict:
-#             for initial_tform in tform_dict["initial"]:
-#                 if isinstance(initial_tform, list) is False:
-#                     initial_tform = [initial_tform]
-#
-#                 for tform in initial_tform:
-#                     image = transform_2d_image(image, [tform])
-#
-#             tform_dict.pop("initial", None)
-#
-#         for k, v in tform_dict.items():
-#             if writer == "zarr" and k == list(tform_dict.keys())[-1]:
-#                 image = transform_2d_image(
-#                     image,
-#                     v,
-#                     writer="zarr",
-#                     zarr_store_dir=im_tform_kwargs["zarr_store_dir"],
-#                     channel_names=im_tform_kwargs["channel_names"],
-#                     channel_colors=im_tform_kwargs["channel_colors"],
-#                 )
-#             else:
-#                 image = transform_2d_image(image, v)
-#
-#     return image
-
-
-#
-# def transform_2d_image(
-#     image, transformation_maps, writer="sitk", **zarr_kwargs
-# ):
-#     """
-#     Transform 2D images with multiple models and return the transformed image
-#     or write the transformed image to disk as a .tif file.
-#     Multichannel or multicomponent images (RGB) have to be transformed a single channel at a time
-#     This function takes care of performing those transformations and reconstructing the image in the same
-#     data type as the input
-#     Parameters
-#     ----------
-#     image : SimpleITK.Image
-#         Image to be transformed
-#     transformation_maps : list
-#         list of SimpleElastix ParameterMaps to used for transformation
-#     Returns
-#     -------
-#     Transformed SimpleITK.Image
-#     """
-#     if transformation_maps is not None:
-#
-#         try:
-#             tfx = sitk.TransformixImageFilter()
-#         except AttributeError:
-#             tfx = sitk.SimpleTransformix()
-#
-#         # TODO: add mask cropping here later
-#
-#         #     print("mask cropping")
-#         #     tmap = sitk.ReadParameterFile(transformation_maps[0])
-#         #     x_min = int(float(tmap["MinimumX"][0]))
-#         #     x_max = int(float(tmap["MaximumX"][0]))
-#         #     y_min = int(float(tmap["MinimumY"][0]))
-#         #     y_max = int(float(tmap["MaximumY"][0]))
-#         #     image = image[x_min:x_max, y_min:y_max]
-#         #     origin = np.repeat(0, len(image.GetSize()))
-#         #     image.SetOrigin(tuple([int(i) for i in origin]))
-#
-#         # else:
-#         for idx, tmap in enumerate(transformation_maps):
-#             if isinstance(tmap, str):
-#                 tmap = sitk.ReadParameterFile(tmap)
-#
-#             if idx == 0:
-#                 tmap["InitialTransformParametersFileName"] = (
-#                     "NoInitialTransform",
-#                 )
-#                 tfx.SetTransformParameterMap(tmap)
-#             else:
-#                 tmap["InitialTransformParametersFileName"] = (
-#                     "NoInitialTransform",
-#                 )
-#
-#                 tfx.AddTransformParameterMap(tmap)
-#         tfx.LogToConsoleOn()
-#         tfx.LogToFileOff()
-#     else:
-#         tfx = None
-#
-#     # if tfx is None:
-#     #     xy_final_size = np.array(image.GetSize(), dtype=np.uint32)
-#     # else:
-#     #     xy_final_size = np.array(
-#     #         transformation_maps[-1]["Size"], dtype=np.uint32
-#     #     )
-#
-#     if writer == "sitk" or writer is None:
-#         return transform_image_to_sitk(image, tfx)
-#     elif writer == "zarr":
-#         return
-#     else:
-#         raise ValueError("writer type {} not recognized".format(writer))
-#
