@@ -6,26 +6,17 @@ import numpy as np
 import SimpleITK as sitk
 from tifffile import TiffWriter
 
-from wsireg.reg_images import RegImage
-from wsireg.reg_transform_seq import RegTransformSeq
+from wsireg.reg_images.reg_image import RegImage
+from wsireg.reg_transforms.reg_transform_seq import RegTransformSeq
 from wsireg.utils.im_utils import (
     format_channel_names,
     get_pyramid_info,
     prepare_ome_xml_str,
 )
-from wsireg.utils.tile_image_transform import (
-    add_tile_location_on_moving,
-    compute_sub_res,
-    determine_moving_tile_padding,
-    generate_tile_coords,
-    itk_transform_tiles,
-    subres_zarr_to_tiles,
-    tile_pad_output_size,
-    zarr_to_tiles,
-)
 
 
 class OmeTiffWriter:
+
     x_size: Optional[int] = None
     y_size: Optional[int] = None
     y_spacing: Optional[Union[int, float]] = None
@@ -43,27 +34,64 @@ class OmeTiffWriter:
         reg_image: RegImage,
         reg_transform_seq: Optional[RegTransformSeq] = None,
     ):
+        """
+        Class for managing writing images to OME-TIFF.
+
+        Parameters
+        ----------
+        reg_image: RegImage
+            RegImage to be transformed
+        reg_transform_seq: RegTransformSeq or None
+            Registration transformation sequence from wsireg to transform image
+
+        Attibutes
+        ---------
+        x_size: int
+            Size of the output image after transformation in x
+        y_size: int
+            Size of the output image after transformation in y
+        y_spacing: float
+            Pixel spacing in microns after transformation in y
+        x_spacing: float
+            Pixel spacing in microns after transformation in x
+        tile_size: int
+            Size of tiles to be written
+        pyr_levels: list of tuples of int:
+            Size of downsampled images in pyramid
+        n_pyr_levels: int
+            Number of downsamples in pyramid
+        PhysicalSizeY: float
+            physical size of image in micron for OME-TIFF in Y
+        PhysicalSizeX: float
+            physical size of image in micron for OME-TIFF in X
+        subifds: int
+            Number of sub-resolutions for pyramidal OME-TIFF
+        compression: str
+            tifffile string to pass to compression argument, defaults to "deflate" for minisblack
+            and "jpeg" for RGB type images
+
+        """
         self.reg_image = reg_image
         self.reg_transform_seq = reg_transform_seq
 
     def _prepare_image_info(
         self,
-        image_name,
+        image_name: str,
         reg_transform_seq: Optional[RegTransformSeq] = None,
-        write_pyramid=True,
-        tile_size=512,
-        compression="default",
-        by_tile=False,
-    ):
+        write_pyramid: bool = True,
+        tile_size: int = 512,
+        compression: Optional[str] = "default",
+    ) -> None:
+        """Get image info and OME-XML"""
 
         if reg_transform_seq:
             self.x_size, self.y_size = reg_transform_seq.output_size
             self.x_spacing, self.y_spacing = reg_transform_seq.output_spacing
         else:
             self.y_size, self.x_size = (
-                (self.reg_image.im_dims[0], self.reg_image.im_dims[1])
+                (self.reg_image.shape[0], self.reg_image.shape[1])
                 if self.reg_image.is_rgb
-                else (self.reg_image.im_dims[1], self.reg_image.im_dims[2])
+                else (self.reg_image.shape[1], self.reg_image.shape[2])
             )
             self.y_spacing, self.x_spacing = None, None
 
@@ -75,11 +103,6 @@ class OmeTiffWriter:
         ):
             self.tile_size = self.tile_size // 2
 
-        if by_tile:
-            self.y_size, self.x_size = tile_pad_output_size(
-                self.y_size, self.x_size, tile_size=self.tile_size
-            )
-
         self.pyr_levels, _ = get_pyramid_info(
             self.y_size, self.x_size, self.reg_image.n_ch, self.tile_size
         )
@@ -120,12 +143,41 @@ class OmeTiffWriter:
 
     def write_image_by_plane(
         self,
-        image_name,
-        output_dir="",
-        write_pyramid=True,
-        tile_size=512,
-        compression="default",
-    ):
+        image_name: str,
+        output_dir: Union[Path, str] = "",
+        write_pyramid: bool = True,
+        tile_size: int = 512,
+        compression: Optional[str] = "default",
+    ) -> str:
+        """
+        Write OME-TIFF image plane-by-plane to disk. WsiReg compatible RegImages all
+        have methods to read an image channel-by-channel, thus each channel is read, transformed, and written to
+        reduce memory during write.
+        RGB images may run large memory footprints as they are interleaved before write, for RGB images,
+        using the `OmeTiledTiffWriter` is recommended.
+
+        Parameters
+        ----------
+        image_name: str
+            Name to be written WITHOUT extension
+            for example if image_name = "cool_image" the file
+            would be "cool_image.ome.tiff"
+        output_dir: Path or str
+            Directory where the image will be saved
+        write_pyramid: bool
+            Whether to write the OME-TIFF with sub-resolutions or not
+        tile_size: int
+            What size to write OME-TIFF tiles to disk
+        compression: str
+            tifffile string to pass to compression argument, defaults to "deflate" for minisblack
+            and "jpeg" for RGB type images
+
+        Returns
+        -------
+        output_file_name: str
+            File path to the written OME-TIFF
+
+        """
 
         output_file_name = str(Path(output_dir) / f"{image_name}.ome.tiff")
         self._prepare_image_info(
@@ -141,16 +193,16 @@ class OmeTiffWriter:
         print(f"saving to {output_file_name}")
         with TiffWriter(output_file_name, bigtiff=True) as tif:
             if self.reg_image.reader == "sitk":
-                self.reg_image.read_full_image()
+                self.reg_image._read_full_image()
+
             for channel_idx in range(self.reg_image.n_ch):
                 print(f"transforming : {channel_idx}")
-                if self.reg_image.reader != "sitk":
-                    image = self.reg_image.read_single_channel(channel_idx)
-                    image = np.squeeze(image)
-                    image = sitk.GetImageFromArray(image)
-                    image.SetSpacing(
-                        (self.reg_image.image_res, self.reg_image.image_res)
-                    )
+                image = self.reg_image.read_single_channel(channel_idx)
+                image = np.squeeze(image)
+                image = sitk.GetImageFromArray(image)
+                image.SetSpacing(
+                    (self.reg_image.image_res, self.reg_image.image_res)
+                )
 
                 if self.reg_transform_seq:
                     image = self.reg_transform_seq.resampler.Execute(image)
@@ -239,169 +291,3 @@ class OmeTiffWriter:
                         )
                         tif.write(rgb_im_data, **options, subfiletype=1)
         return output_file_name
-
-    def write_image_by_tile(
-        self,
-        image_name,
-        output_dir="",
-        write_pyramid=True,
-        tile_size=512,
-        compression="default",
-        tile_padding=None,
-        use_multiprocessing=False,
-    ):
-
-        output_file_name = str(Path(output_dir) / f"{image_name}.ome.tiff")
-
-        self._prepare_image_info(
-            image_name,
-            reg_transform_seq=self.reg_transform_seq,
-            write_pyramid=write_pyramid,
-            tile_size=tile_size,
-            compression=compression,
-            by_tile=True,
-        )
-
-        # create per tile coordinate information for fixed
-        # and moving
-        tile_coordinate_data = generate_tile_coords(
-            (self.x_size, self.y_size),
-            (self.PhysicalSizeX, self.PhysicalSizeY),
-            tile_size=self.tile_size,
-        )
-        # itk_transforms = [
-        #     t.itk_transform for t in self.reg_transform_seq.reg_transforms
-        # ]
-
-        tile_coordinate_data = add_tile_location_on_moving(
-            self.reg_transform_seq.reg_transforms_itk_order,
-            tile_coordinate_data,
-            self.reg_image.image_res,
-        )
-
-        if tile_padding is None:
-            tile_padding = determine_moving_tile_padding(
-                self.reg_transform_seq.reg_transforms_itk_order
-            )
-
-        print(f"saving to {output_file_name}")
-        with TiffWriter(output_file_name, bigtiff=True) as tif:
-            options = dict(
-                tile=(self.tile_size, self.tile_size),
-                compression=self.compression,
-                photometric="rgb" if self.reg_image.is_rgb else "minisblack",
-                metadata=None,
-            )
-            if self.reg_image.is_rgb:
-                resampled_zarray = itk_transform_tiles(
-                    self.reg_image,
-                    self.y_size,
-                    self.x_size,
-                    tile_coordinate_data,
-                    self.reg_transform_seq.composite_transform,
-                    tile_size=self.tile_size,
-                    tile_padding=tile_padding,
-                    use_multiprocessing=use_multiprocessing,
-                )
-                print(
-                    f"writing base layer RGB - shape: {resampled_zarray.shape}"
-                )
-                description = self.omexml
-                tif.write(
-                    zarr_to_tiles(
-                        resampled_zarray,
-                        tile_coordinate_data,
-                        self.reg_image.is_rgb,
-                    ),
-                    subifds=self.subifds,
-                    description=description,
-                    shape=(self.y_size, self.x_size, 3),
-                    dtype=resampled_zarray.dtype,
-                    **options,
-                )
-                if write_pyramid:
-                    for pyr_idx in range(1, self.n_pyr_levels):
-
-                        resampled_zarray_subres, orig_shape = compute_sub_res(
-                            self.reg_image,
-                            resampled_zarray,
-                            2**pyr_idx,
-                            self.tile_size,
-                            self.reg_image.is_rgb,
-                        )
-                        print(f"pyr {pyr_idx} : RGB-shape: {orig_shape}")
-
-                        tif.write(
-                            subres_zarr_to_tiles(
-                                resampled_zarray_subres,
-                                self.tile_size,
-                                self.reg_image.is_rgb,
-                            ),
-                            shape=orig_shape,
-                            dtype=resampled_zarray_subres.dtype,
-                            **options,
-                            subfiletype=1,
-                        )
-                try:
-                    resampled_zarray.store.clear()
-                except FileNotFoundError:
-                    pass
-            else:
-                for channel_idx in range(self.reg_image.n_ch):
-                    resampled_zarray = itk_transform_tiles(
-                        self.reg_image,
-                        self.y_size,
-                        self.x_size,
-                        tile_coordinate_data,
-                        self.reg_transform_seq.composite_transform,
-                        ch_idx=channel_idx,
-                        tile_size=self.tile_size,
-                        tile_padding=tile_padding,
-                        use_multiprocessing=use_multiprocessing,
-                    )
-
-                    description = self.omexml if channel_idx == 0 else None
-                    print(
-                        f"writing channel {channel_idx} - shape: {resampled_zarray.shape}"
-                    )
-                    tif.write(
-                        zarr_to_tiles(
-                            resampled_zarray,
-                            tile_coordinate_data,
-                            self.reg_image.is_rgb,
-                        ),
-                        subifds=self.subifds,
-                        description=description,
-                        shape=(self.y_size, self.x_size),
-                        dtype=resampled_zarray.dtype,
-                        **options,
-                    )
-                    if write_pyramid:
-                        for pyr_idx in range(1, self.n_pyr_levels):
-                            (
-                                resampled_zarray_subres,
-                                orig_shape,
-                            ) = compute_sub_res(
-                                self.reg_image,
-                                resampled_zarray,
-                                2**pyr_idx,
-                                self.tile_size,
-                                self.reg_image.is_rgb,
-                            )
-
-                            tif.write(
-                                subres_zarr_to_tiles(
-                                    resampled_zarray_subres,
-                                    self.tile_size,
-                                    self.reg_image.is_rgb,
-                                ),
-                                shape=orig_shape,
-                                dtype=resampled_zarray_subres.dtype,
-                                **options,
-                                subfiletype=1,
-                            )
-                    try:
-                        resampled_zarray.store.clear()
-                    except FileNotFoundError:
-                        pass
-        return output_file_name
