@@ -6,22 +6,26 @@ from typing import Any, Dict, List, Optional, Tuple, Union
 from warnings import warn
 
 import numpy as np
-import SimpleITK as sitk
 import yaml
 
 from wsireg.parameter_maps.preprocessing import ImagePreproParams
 from wsireg.parameter_maps.reg_model import RegModel
-from wsireg.reg_images import RegImage, MergeRegImage
+from wsireg.reg_images import MergeRegImage
+from wsireg.reg_images.reg_image import RegImage
+
 from wsireg.reg_images.loader import reg_image_loader
 from wsireg.reg_shapes import RegShapes
-from wsireg.reg_transform import RegTransform
-from wsireg.reg_transform_seq import RegTransformSeq
+from wsireg.reg_transforms import RegTransform
+from wsireg.reg_transforms import RegTransformSeq
 from wsireg.utils.config_utils import parse_check_reg_config
 from wsireg.utils.im_utils import ARRAYLIKE_CLASSES
+from wsireg.utils.output_utils import (
+    read_elastix_iteration_dir,
+    read_elastix_transform_dir,
+    write_iteration_plots,
+)
 from wsireg.utils.reg_utils import (
     _prepare_reg_models,
-    json_to_pmap_dict,
-    pmap_dict_to_json,
     register_2d_images_itkelx,
     sitk_pmap_to_dict,
 )
@@ -29,12 +33,10 @@ from wsireg.utils.shape_utils import invert_nonrigid_transforms
 from wsireg.utils.tform_utils import identity_elx_transform
 from wsireg.writers.merge_ome_tiff_writer import MergeOmeTiffWriter
 from wsireg.writers.ome_tiff_writer import OmeTiffWriter
+from wsireg.writers.tiled_ome_tiff_writer import OmeTiffTiledWriter
 
 
 class WsiReg2D(object):
-    project_name: Optional[str] = None
-    output_dir: Optional[Union[str, Path]] = None
-    image_cache: Optional[Path] = None
     """
     Class to define a 2D registration graph and execute the registrations and transformations of the graph
 
@@ -52,9 +54,14 @@ class WsiReg2D(object):
 
     Attributes
     ----------
+    project_name: str
+        Global project name, will be appended to all output files and folders
+    output_dir: Path
+        Directory where registration data will be stored
+    image_cache: Path
+        Directory where images are cached after preprocessing
     modalities: dict
         dictionary of modality information (file path, spatial res., preprocessing), defines a graph node
-
     modalities: list
         list of all modality names
 
@@ -82,6 +89,11 @@ class WsiReg2D(object):
     attachment_shapes: dict
         shape data attached to a modality to be transformed along the graph
 
+    registration_iter_data: Dict[str,Dict[int, Dict[int, Dict[str, np.ndarray]]]]
+        elastix data for each iteration in the registration sorted by transformation model and resolution
+
+    registration_tform_data: Dict[str, Dict[int, Dict[int, Dict[str, str]]]]
+        elastix transform data for each resolution sorted by transformation model and resolution
 
     """
 
@@ -92,6 +104,9 @@ class WsiReg2D(object):
         cache_images: bool = True,
         config: Optional[Union[str, Path]] = None,
     ):
+        self.project_name: Optional[str] = None
+        self.output_dir: Optional[Union[str, Path]] = None
+        self.image_cache: Optional[Path] = None
 
         self.setup_project_output(project_name, output_dir)
 
@@ -110,6 +125,9 @@ class WsiReg2D(object):
         self._preprocessed_image_spacings: Dict[
             str, Union[Tuple[int, int], Tuple[float, float]]
         ] = dict()
+        self._transformed_shapes_spacings: Dict[
+            str, Tuple[float, float]
+        ] = dict()
 
         self.n_modalities: Optional[int] = None
         self.n_registrations: Optional[int] = None
@@ -122,6 +140,13 @@ class WsiReg2D(object):
         self.merge_modalities = dict()
         self.original_size_transforms = dict()
 
+        self.registration_iter_data: Dict[
+            str, Dict[int, Dict[int, Dict[str, np.ndarray]]]
+        ] = dict()
+        self.registration_tform_data: Dict[
+            str, Dict[int, Dict[int, Dict[str, str]]]
+        ] = dict()
+
         if config:
             self.add_data_from_config(config)
 
@@ -130,6 +155,18 @@ class WsiReg2D(object):
         project_name: Optional[str] = None,
         output_dir: Optional[Union[str, Path]] = None,
     ) -> None:
+        """
+        Set up the project directory and image cache.
+
+        Parameters
+        ----------
+        project_name: str
+            Project name will prefix all output files and directories
+        output_dir: str
+            Directory where registration data will be stored
+
+
+        """
         if project_name is None:
             self.project_name = 'RegProj'
         else:
@@ -143,7 +180,9 @@ class WsiReg2D(object):
         self.image_cache = self.output_dir / ".imcache_{}".format(project_name)
 
     @property
-    def modalities(self):
+    def modalities(self) -> Dict[str, Any]:
+        """Image modality information stored as a dictionary where keys are images and values
+        are all image modality related information."""
         return self._modalities
 
     @modalities.setter
@@ -153,6 +192,8 @@ class WsiReg2D(object):
 
     @property
     def shape_sets(self):
+        """Shape data stored as a dictionary where keys are shape sets and values
+        are all shape related information."""
         return self._shape_sets
 
     @shape_sets.setter
@@ -160,7 +201,8 @@ class WsiReg2D(object):
         self._shape_sets.update(shape_set)
 
     @property
-    def shape_set_names(self):
+    def shape_set_names(self) -> List[str]:
+        """Name of all shape sets"""
         return self._shape_set_names
 
     @shape_set_names.setter
@@ -168,7 +210,8 @@ class WsiReg2D(object):
         self._shape_set_names.append(shape_set_name)
 
     @property
-    def modality_names(self):
+    def modality_names(self) -> List[str]:
+        """List of all the modality names."""
         return self._modality_names
 
     @modality_names.setter
@@ -207,8 +250,9 @@ class WsiReg2D(object):
         channel_colors: List[str]
             channels colors for OME-TIFF (not implemented)
         mask: Union[str, Path, np.ndarray]
-            path to binary mask (>0 is in) image for registration and/or cropping
-        preprocessing :
+            path to binary mask (>0 is in) image for registration and/or cropping or a geoJSON with shapes
+            that will be processed to a binary mask
+        preprocessing: ImagePreproParams
             preprocessing parameters for the modality for registration. Registration images should be a xy single plane
             so many modalities (multi-channel, RGB) must "create" a single channel.
             Defaults: multi-channel images -> max intensity project image
@@ -216,6 +260,8 @@ class WsiReg2D(object):
         output_res : Union[Tuple[int,int], Tuple[float,float]]
             change output spacing/resolution when resampling images, default will be the spacing
             of the final target image
+        prepro_dict: dict
+            deprecated version kept temporarily
         """
         if modality_name in self._modality_names:
             raise ValueError(
@@ -370,7 +416,9 @@ class WsiReg2D(object):
         )
 
     @property
-    def reg_paths(self):
+    def reg_paths(self) -> Dict[str, List[str]]:
+        """Dictionary of paths between modalities by modality name.
+        Keys are modality name and the values are a list of modalities it passes through."""
         return self._reg_paths
 
     @reg_paths.setter
@@ -389,11 +437,29 @@ class WsiReg2D(object):
             )
         else:
             self._reg_paths.update({src_modality: [tgt_modality]})
+        if override_prepro:
+            if override_prepro.get("source"):
+                source_override = ImagePreproParams(
+                    **override_prepro.get("source")
+                )
+            else:
+                source_override = None
+
+            if override_prepro.get("target"):
+                target_override = ImagePreproParams(
+                    **override_prepro.get("target")
+                )
+            else:
+                target_override = None
+        else:
+            source_override = None
+            target_override = None
 
         self.reg_graph_edges = {
             'modalities': {'source': src_modality, 'target': thru_modality},
             'params': reg_params,
-            "override_prepro": override_prepro,
+            "source_override": source_override,
+            "target_override": target_override,
         }
         self.transform_paths = self._reg_paths
 
@@ -403,7 +469,7 @@ class WsiReg2D(object):
         tgt_modality_name: str,
         thru_modality: Optional[str] = None,
         reg_params: Union[str, RegModel, List[str], List[RegModel]] = [
-            RegModel.rigid
+            "rigid"
         ],
         override_prepro: dict = {"source": None, "target": None},
     ):
@@ -418,9 +484,10 @@ class WsiReg2D(object):
             modality that has been added to graph that is being aligned to
         thru_modality: str
             modality that has been added to graph by which another should be run through
-        reg_params:
-            SimpleElastix registration parameters
-        override_prepro:
+        reg_params: list of RegModel/str or str
+            Elastix registration parameters, from RegModel or as a string corresponding to one of the parameter
+            maps enumerated in wsireg.parameter_maps.reg_params.RegModel
+        override_prepro: dict
             set specific preprocessing for a given registration edge for the source or target image that will override
             the set modality preprocessing FOR THIS REGISTRATION ONLY.
         """
@@ -503,125 +570,6 @@ class WsiReg2D(object):
                     return extended_path
         return None
 
-    def _check_cache_modality(self, modality_name: str):
-        cache_im_fp = self.image_cache / "{}_prepro.tiff".format(modality_name)
-        cache_transform_fp = cache_im_fp.parent / "{}_init_tforms.json".format(
-            cache_im_fp.stem
-        )
-        cache_osize_tform_fp = (
-            self.image_cache
-            / "{}_orig_size_tform.json".format(cache_im_fp.stem)
-        )
-
-        if cache_im_fp.exists() is True:
-            im_fp = str(cache_im_fp)
-            im_from_cache = True
-        else:
-            im_fp = self.modalities[modality_name]["image_filepath"]
-            im_from_cache = False
-
-        if cache_transform_fp.exists() is True:
-            im_initial_transforms = [json_to_pmap_dict(cache_transform_fp)]
-        else:
-            im_initial_transforms = None
-
-        if cache_osize_tform_fp.exists() is True:
-            osize_tform = json_to_pmap_dict(cache_osize_tform_fp)
-        else:
-            osize_tform = None
-
-        return im_fp, im_initial_transforms, im_from_cache, osize_tform
-
-    def _prepare_modality(
-        self, modality_name: str, reg_edge: dict, src_or_tgt: str
-    ) -> Tuple:
-        mod_data = self.modalities[modality_name].copy()
-
-        if reg_edge.get("override_prepro") is not None:
-            override_preprocessing = reg_edge.get("override_prepro")[
-                src_or_tgt
-            ]
-        else:
-            override_preprocessing = None
-
-        if override_preprocessing is not None:
-            mod_data["preprocessing"] = override_preprocessing
-
-            return (
-                mod_data["image_filepath"],
-                mod_data["image_res"],
-                mod_data["preprocessing"],
-                None,
-                mod_data["mask"],
-                None,
-            )
-        else:
-
-            (
-                mod_data["image_filepath"],
-                mod_data["transforms"],
-                im_from_cache,
-                original_size_transform,
-            ) = self._check_cache_modality(modality_name)
-
-            if im_from_cache:
-                if mod_data["preprocessing"].use_mask is False:
-                    mod_data["mask"] = None
-                if mod_data["preprocessing"].downsampling > 1:
-                    image_res = (
-                        mod_data["image_res"]
-                        * mod_data["preprocessing"].downsampling
-                    )
-                else:
-                    image_res = mod_data["image_res"]
-                mod_data["preprocessing"] = None
-            else:
-                image_res = mod_data["image_res"]
-
-            return (
-                mod_data["image_filepath"],
-                image_res,
-                mod_data["preprocessing"],
-                mod_data["transforms"],
-                mod_data["mask"],
-                original_size_transform,
-            )
-
-    def _cache_images(self, modality_name: str, reg_image: RegImage) -> None:
-
-        cache_im_fp = self.image_cache / "{}_prepro.tiff".format(modality_name)
-
-        cache_transform_fp = self.image_cache / "{}_init_tforms.json".format(
-            cache_im_fp.stem
-        )
-
-        cache_osize_tform_fp = (
-            self.image_cache
-            / "{}_orig_size_tform.json".format(cache_im_fp.stem)
-        )
-        if cache_im_fp.is_file() is False:
-            sitk.WriteImage(reg_image.reg_image, str(cache_im_fp), True)
-
-        if reg_image.mask is not None:
-            cache_mask_im_fp = self.image_cache / "{}_prepro_mask.tiff".format(
-                modality_name
-            )
-            if cache_mask_im_fp.is_file() is False:
-                sitk.WriteImage(reg_image.mask, str(cache_mask_im_fp), True)
-
-        if cache_transform_fp.is_file() is False:
-            pmap_dict_to_json(
-                reg_image.pre_reg_transforms, str(cache_transform_fp)
-            )
-
-        if (
-            cache_osize_tform_fp.is_file() is False
-            and reg_image.original_size_transform is not None
-        ):
-            pmap_dict_to_json(
-                reg_image.original_size_transform, str(cache_osize_tform_fp)
-            )
-
     def _find_nonreg_modalities(self) -> None:
         registered_modalities = [
             edge.get("modalities").get("source")
@@ -639,7 +587,11 @@ class WsiReg2D(object):
 
         return non_reg_modalities
 
-    def save_config(self, registered=False):
+    def save_config(
+        self,
+        output_file_path: Optional[Union[str, Path]] = None,
+        registered: bool = False,
+    ):
         ts = time.strftime('%Y%m%d-%H%M%S')
         status = "registered" if registered is True else "setup"
 
@@ -672,6 +624,10 @@ class WsiReg2D(object):
         for mod, data in modalities_out.items():
             if isinstance(data["image_filepath"], ARRAYLIKE_CLASSES):
                 data["image_filepath"] = "ArrayLike"
+            if isinstance(data["preprocessing"], ImagePreproParams):
+                data["preprocessing"] = deepcopy(data["preprocessing"]).dict(
+                    exclude_none=True, exclude_defaults=True
+                )
 
         config = {
             "project_name": self.project_name,
@@ -693,12 +649,13 @@ class WsiReg2D(object):
             else None,
         }
 
-        output_path = (
-            self.output_dir
-            / f"{ts}-{self.project_name}-configuration-{status}.yaml"
-        )
+        if not output_file_path:
+            output_file_path = (
+                self.output_dir
+                / f"{ts}-{self.project_name}-configuration-{status}.yaml"
+            )
 
-        with open(str(output_path), "w") as f:
+        with open(str(output_file_path), "w") as f:
             yaml.dump(config, f, sort_keys=False)
 
     def register_images(self, parallel=False):
@@ -723,47 +680,74 @@ class WsiReg2D(object):
                 src_name = reg_edge["modalities"]["source"]
                 tgt_name = reg_edge["modalities"]["target"]
 
-                (
-                    src_reg_image_fp,
-                    src_res,
-                    src_prepro,
-                    src_transforms,
-                    src_mask,
-                    src_original_size_transform,
-                ) = self._prepare_modality(src_name, reg_edge, "source")
-
-                (
-                    tgt_reg_image_fp,
-                    tgt_res,
-                    tgt_prepro,
-                    tgt_transforms,
-                    tgt_mask,
-                    tgt_original_size_transform,
-                ) = self._prepare_modality(tgt_name, reg_edge, "target")
+                src_mod_data = self.modalities[src_name].copy()
+                tgt_mod_data = self.modalities[tgt_name].copy()
 
                 src_reg_image = reg_image_loader(
-                    src_reg_image_fp,
-                    src_res,
-                    preprocessing=src_prepro,
-                    pre_reg_transforms=src_transforms,
-                    mask=src_mask,
+                    src_mod_data["image_filepath"],
+                    src_mod_data["image_res"],
+                    preprocessing=src_mod_data["preprocessing"],
+                    mask=src_mod_data["mask"],
                 )
 
                 tgt_reg_image = reg_image_loader(
-                    tgt_reg_image_fp,
-                    tgt_res,
-                    preprocessing=tgt_prepro,
-                    pre_reg_transforms=tgt_transforms,
-                    mask=tgt_mask,
+                    tgt_mod_data["image_filepath"],
+                    tgt_mod_data["image_res"],
+                    preprocessing=tgt_mod_data["preprocessing"],
+                    mask=tgt_mod_data["mask"],
                 )
 
-                src_reg_image.read_reg_image()
-                tgt_reg_image.read_reg_image()
+                src_override_prepro = reg_edge.get("source_override")
+                tgt_override_prepro = reg_edge.get("target_override")
+
+                src_cached = src_reg_image.check_cache_preprocessing(
+                    self.image_cache, src_name
+                )
+                tgt_cached = src_reg_image.check_cache_preprocessing(
+                    self.image_cache, tgt_name
+                )
+
+                if not src_cached and not src_override_prepro:
+                    src_reg_image.read_reg_image()
+                    if self.cache_images:
+                        src_reg_image.cache_image_data(
+                            self.image_cache, src_name, check=False
+                        )
+                elif src_override_prepro:
+                    if src_override_prepro:
+                        src_reg_image._preprocessing = src_override_prepro
+                    src_reg_image.read_reg_image()
+                    if self.cache_images:
+                        src_reg_image.cache_image_data(
+                            self.image_cache,
+                            f"{src_name}-{tgt_name}-override",
+                            check=False,
+                        )
+                else:
+                    src_reg_image.load_from_cache(self.image_cache, src_name)
+
+                if not tgt_cached and not tgt_override_prepro:
+                    tgt_reg_image.read_reg_image()
+                    if self.cache_images:
+                        tgt_reg_image.cache_image_data(
+                            self.image_cache, tgt_name, check=False
+                        )
+                elif tgt_override_prepro:
+                    if tgt_override_prepro:
+                        tgt_reg_image._preprocessing = tgt_override_prepro
+                    tgt_reg_image.read_reg_image()
+                    if self.cache_images:
+                        src_reg_image.cache_image_data(
+                            self.image_cache,
+                            f"{tgt_name}-{src_name}-override",
+                            check=False,
+                        )
+                else:
+                    tgt_reg_image.load_from_cache(self.image_cache, tgt_name)
 
                 self._preprocessed_image_spacings.update(
                     {src_name: src_reg_image.reg_image.GetSpacing()}
                 )
-
                 self._preprocessed_image_spacings.update(
                     {tgt_name: tgt_reg_image.reg_image.GetSpacing()}
                 )
@@ -774,30 +758,6 @@ class WsiReg2D(object):
                     {tgt_name: tgt_reg_image.reg_image.GetSize()}
                 )
 
-                if (
-                    tgt_original_size_transform is None
-                    and tgt_reg_image.original_size_transform is not None
-                ):
-                    tgt_original_size_transform = (
-                        tgt_reg_image.original_size_transform
-                    )
-
-                if self.cache_images is True:
-                    if reg_edge.get("override_prepro") is not None:
-                        if (
-                            reg_edge.get("override_prepro").get("source")
-                            is None
-                        ):
-                            self._cache_images(src_name, src_reg_image)
-                        if (
-                            reg_edge.get("override_prepro").get("target")
-                            is None
-                        ):
-                            self._cache_images(tgt_name, tgt_reg_image)
-                    else:
-                        self._cache_images(src_name, src_reg_image)
-                        self._cache_images(tgt_name, tgt_reg_image)
-
                 reg_params = reg_edge["params"]
 
                 output_path = (
@@ -822,10 +782,7 @@ class WsiReg2D(object):
 
                 reg_tforms = [sitk_pmap_to_dict(tf) for tf in reg_tforms]
 
-                if src_transforms is not None:
-                    initial_transforms = src_transforms[0]
-                else:
-                    initial_transforms = src_reg_image.pre_reg_transforms
+                initial_transforms = src_reg_image.pre_reg_transforms
 
                 if initial_transforms:
                     initial_transforms_rt = [
@@ -848,13 +805,26 @@ class WsiReg2D(object):
                 }
 
                 self.original_size_transforms.update(
-                    {tgt_name: tgt_original_size_transform}
+                    {tgt_name: tgt_reg_image.original_size_transform}
                 )
 
                 reg_edge["registered"] = True
+                data_key = f'{reg_edge["modalities"]["source"]}_to_{reg_edge["modalities"]["target"]}'
+
+                all_transform_data = read_elastix_transform_dir(output_path)
+                all_iteration_data = read_elastix_iteration_dir(output_path)
+                write_iteration_plots(
+                    all_iteration_data, data_key, output_path
+                )
+
+                self.registration_iter_data.update(
+                    {data_key: all_iteration_data}
+                )
+                self.registration_tform_data.update(
+                    {data_key: all_transform_data}
+                )
 
         self.transformations = self.reg_graph_edges
-        # self.save_config(registered=True)
 
     @property
     def transformations(self):
@@ -941,20 +911,27 @@ class WsiReg2D(object):
             or im_data["preprocessing"].crop_to_mask_bbox
             or im_data["preprocessing"].mask_bbox
         ):
-            _, im_initial_transforms, _, _ = self._check_cache_modality(
-                modality_key
+
+            im_initial_transforms = RegImage.load_orignal_size_transform(
+                self.image_cache, modality_key
             )
 
-            transformations = RegTransformSeq(
-                [RegTransform(t) for t in im_initial_transforms[0]],
-                transform_seq_idx=[
-                    idx for idx in range(len(im_initial_transforms))
-                ],
-            )
+            if any(im_initial_transforms):
+                transformations = RegTransformSeq(
+                    [RegTransform(t) for t in im_initial_transforms[0]],
+                    transform_seq_idx=[
+                        idx for idx in range(len(im_initial_transforms))
+                    ],
+                )
 
         if to_original_size and self.original_size_transforms[modality_key]:
+
+            o_size_tform = self.original_size_transforms[modality_key]
+            if isinstance(o_size_tform, list):
+                o_size_tform = o_size_tform[0]
+
             orig_size_rt = RegTransformSeq(
-                RegTransform(self.original_size_transforms[modality_key]),
+                RegTransform(o_size_tform),
                 transform_seq_idx=[0],
             )
             if transformations:
@@ -1035,6 +1012,9 @@ class WsiReg2D(object):
             original_size_transform = self.original_size_transforms[
                 final_modality
             ]
+            if isinstance(original_size_transform, list):
+                original_size_transform = original_size_transform[0]
+
             orig_size_rt = RegTransformSeq(
                 RegTransform(original_size_transform), transform_seq_idx=[0]
             )
@@ -1078,6 +1058,9 @@ class WsiReg2D(object):
             file_writer == "ome.tiff-bytile"
             and ometiffwriter.reg_image.reader not in ["czi", "sitk"]
         ):
+            ometiffwriter = OmeTiffTiledWriter(
+                tfregimage, reg_transform_seq=transformations
+            )
             im_fp = ometiffwriter.write_image_by_tile(
                 output_path.stem,
                 output_dir=str(self.output_dir),
@@ -1158,7 +1141,7 @@ class WsiReg2D(object):
         )
 
         merge_ometiffwriter = MergeOmeTiffWriter(
-            merge_regimage, reg_seq_transforms=transformations
+            merge_regimage, reg_transform_seqs=transformations
         )
 
         im_fp = merge_ometiffwriter.merge_write_image_by_plane(
@@ -1313,57 +1296,83 @@ class WsiReg2D(object):
         # do inversion before any processing to ensure it is not computed twice
         for set_name, set_data in self.shape_sets.items():
             attachment_modality = set_data["attachment_modality"]
-            invert_nonrigid_transforms(
-                self.transformations[attachment_modality][
-                    "full-transform-seq"
-                ].reg_transforms_itk_order
-            )
+            if attachment_modality in self.transformations.keys():
+                invert_nonrigid_transforms(
+                    self.transformations[attachment_modality][
+                        "full-transform-seq"
+                    ].reg_transforms_itk_order
+                )
+            else:
+                continue
 
         for set_name, set_data in self.shape_sets.items():
             attachment_modality = set_data["attachment_modality"]
-            im_data = self.modalities[attachment_modality]
-
-            final_modality = self.reg_paths[attachment_modality][-1]
-
-            print(
-                "transforming shape set {} associated with {} to {}".format(
-                    set_name, attachment_modality, final_modality
-                )
-            )
-
             rs = RegShapes(
                 set_data["shape_files"], source_res=set_data["image_res"]
             )
 
-            transformations = copy(
-                self.transformations[attachment_modality]["full-transform-seq"]
-            )
+            if attachment_modality in self.transformations.keys():
+                im_data = self.modalities[attachment_modality]
+                final_modality = self.reg_paths[attachment_modality][-1]
 
-            if im_data["output_res"]:
-                transformations.set_output_spacing(im_data["output_res"])
+                print(
+                    "transforming shape set {} associated with {} to {}".format(
+                        set_name, attachment_modality, final_modality
+                    )
+                )
 
+                transformations = copy(
+                    self.transformations[attachment_modality][
+                        "full-transform-seq"
+                    ]
+                )
+
+                if im_data.get("output_res"):
+                    transformations.set_output_spacing(im_data["output_res"])
+
+                else:
+                    output_spacing_target = self.modalities[final_modality][
+                        "image_res"
+                    ]
+                    transformations.set_output_spacing(
+                        (output_spacing_target, output_spacing_target)
+                    )
+
+                self._transformed_shapes_spacings.update(
+                    {set_name: (output_spacing_target, output_spacing_target)}
+                )
+
+                rs.transform_shapes(transformations)
+                output_path = (
+                    self.output_dir
+                    / "{}-{}-{}_to_{}-transformed_shapes.geojson".format(
+                        self.project_name,
+                        set_name,
+                        attachment_modality,
+                        final_modality,
+                    )
+                )
+
+                rs.save_shape_data(output_path, transformed=True)
+                transformed_shapes_fps.append(output_path)
             else:
-                output_spacing_target = self.modalities[final_modality][
+                output_path = (
+                    self.output_dir
+                    / "{}-{}-{}-registered.geojson".format(
+                        self.project_name,
+                        set_name,
+                        attachment_modality,
+                    )
+                )
+
+                output_spacing_target = self.modalities[attachment_modality][
                     "image_res"
                 ]
-                transformations.set_output_spacing(
-                    (output_spacing_target, output_spacing_target)
+                self._transformed_shapes_spacings.update(
+                    {set_name: (output_spacing_target, output_spacing_target)}
                 )
-
-            rs.transform_shapes(transformations)
-
-            output_path = (
-                self.output_dir
-                / "{}-{}-{}_to_{}-transformed_shapes.geojson".format(
-                    self.project_name,
-                    set_name,
-                    attachment_modality,
-                    final_modality,
-                )
-            )
-
-            rs.save_shape_data(output_path, transformed=True)
-            transformed_shapes_fps.append(output_path)
+                rs.save_shape_data(output_path, transformed=False)
+                transformed_shapes_fps.append(output_path)
 
         return transformed_shapes_fps
 
@@ -1526,10 +1535,107 @@ class WsiReg2D(object):
             modality_idx = edge_keys.index(modality)
             self.reg_graph_edges[modality_idx]["registered"] = False
 
+    def _remove_modality_data(self, modality: str) -> None:
+        if modality in self.modality_names:
+            # remove top level
+            self.modalities.pop(modality)
+            self.modality_names.pop(self.modality_names.index(modality))
 
-def main():
-    import argparse
+        to_rm = []
+        for merge_name, merge_mods in self.merge_modalities.items():
+            if modality in merge_mods:
+                to_rm.append(merge_name)
 
+        [self.merge_modalities.pop(k) for k in to_rm]
+
+        self.n_modalities = len(self.modality_names)
+
+        if modality in self.attachment_images.keys():
+            self.attachment_images.pop(modality)
+
+    def _remove_reg_paths(self, modality: str) -> None:
+        # remove registrations
+        if modality in self.reg_paths.keys():
+            self.reg_paths.pop(modality)
+
+        # remove any reg paths including modality as target or thru
+        to_rm = []
+        for k, v in self.reg_paths.items():
+            if modality in v:
+                to_rm.append(k)
+
+        [self.reg_paths.pop(k) for k in to_rm]
+
+        # remove any reg paths including modality as target or thru
+        to_rm = []
+        for k, v in self.transform_paths.items():
+            for reg in v:
+                if reg["source"] == modality or reg["target"] == modality:
+                    to_rm.append(k)
+
+        to_rm = list(np.unique(to_rm))
+
+        edges_to_rm = []
+        for idx, edge in enumerate(self.reg_graph_edges):
+            if (
+                edge["modalities"]["source"] in to_rm
+                or edge["modalities"]["target"] in to_rm
+            ):
+                edges_to_rm.append(idx)
+
+        [self.transform_paths.pop(k) for k in to_rm]
+
+        for idx in sorted(edges_to_rm)[::-1]:
+            self.reg_graph_edges.pop(idx)
+
+        self.n_registrations = len(self._reg_graph_edges)
+
+    def _remove_attachments(self, modality: str):
+        to_rm = []
+        for k, v in self.attachment_images.items():
+            if modality == v:
+                to_rm.append(k)
+
+        [self.attachment_images.pop(k) for k in to_rm]
+        [self.modalities.pop(k) for k in to_rm]
+        [self.modality_names.pop(self.modality_names.index(k)) for k in to_rm]
+
+        self.shape_sets.pop(modality)
+        self.shape_set_names.pop(self.shape_set_names.index(modality))
+
+        self.n_modalities = len(self.modality_names)
+
+    def remove_merge_modality(self, merge_modality: str) -> None:
+        try:
+            self.merge_modalities.pop(merge_modality)
+        except KeyError:
+            warn(f"merge modality {merge_modality} not found")
+
+    def remove_modality(self, modality: str) -> None:
+
+        if modality in self.modality_names:
+            self._remove_modality_data(modality)
+            self._remove_reg_paths(modality)
+
+        elif modality in self.shape_set_names:
+            self._remove_attachments(modality)
+
+        else:
+            warn(
+                f"{modality} not found in modalities: {', '.join(self.modality_names)}"
+                f" or shape sets {self.shape_set_names}. "
+                "It may have already been removed"
+            )
+
+
+def main(
+    graph_configuration: Union[str, Path, WsiReg2D],
+    write_images: bool = True,
+    to_original_size: bool = False,
+    transform_non_reg: bool = True,
+    remove_merged: bool = True,
+    file_writer: str = "ome.tiff",
+):
     def config_to_WsiReg2D(config_filepath):
         reg_config = parse_check_reg_config(config_filepath)
 
@@ -1540,6 +1646,40 @@ def main():
         )
         return reg_graph
 
+    if isinstance(graph_configuration, (str, Path)):
+        reg_graph = config_to_WsiReg2D(config_filepath)
+        reg_graph.add_data_from_config(config_filepath)
+    elif isinstance(graph_configuration, WsiReg2D):
+        reg_graph = graph_configuration
+
+    reg_graph.register_images()
+    reg_graph.save_transformations()
+    output_data = []
+    if write_images:
+        output_images = reg_graph.transform_images(
+            file_writer=file_writer,
+            to_original_size=to_original_size,
+            transform_non_reg=transform_non_reg,
+            remove_merged=remove_merged,
+        )
+    else:
+        output_images = []
+
+    if reg_graph.shape_sets:
+        output_shapes = reg_graph.transform_shapes()
+    else:
+        output_shapes = []
+
+    output_data.extend(output_images)
+    output_data.extend(output_shapes)
+
+    return output_data
+
+
+if __name__ == "__main__":
+    import argparse
+    import sys
+
     parser = argparse.ArgumentParser(
         description='Load Whole Slide Image 2D Registration Graph from configuration file'
     )
@@ -1548,41 +1688,59 @@ def main():
         "config_filepath",
         metavar="C",
         type=str,
-        nargs=1,
         help="full filepath for .yaml configuration file",
     )
     parser.add_argument(
         "--fw",
         type=str,
-        nargs=1,
         help="how to write output registered images: ome.tiff, ome.zarr (default: ome.tiff)",
     )
 
     parser.add_argument('--write_im', dest='write_im', action='store_true')
     parser.add_argument('--no_write_im', dest='write_im', action='store_false')
-    parser.set_defaults(write_im=True)
+
+    parser.add_argument(
+        '--remove_merged', dest='remove_merged', action='store_true'
+    )
+    parser.add_argument(
+        '--write_merge_indiv', dest='remove_merged', action='store_false'
+    )
+
+    parser.add_argument(
+        '--tform_non_reg', dest='transform_non_reg', action='store_true'
+    )
+    parser.add_argument(
+        '--no_tform_non_reg', dest='transform_non_reg', action='store_false'
+    )
+
+    parser.add_argument(
+        '--to_orig_size', dest='to_original_size', action='store_true'
+    )
+    parser.add_argument(
+        '--to_cropped', dest='to_original_size', action='store_false'
+    )
+
+    parser.set_defaults(
+        write_im=True,
+        remove_merged=True,
+        transform_non_reg=True,
+        to_original_size=False,
+    )
 
     args = parser.parse_args()
-    config_filepath = args.config_filepath[0]
+    config_filepath = args.config_filepath
+
     if args.fw is None:
         file_writer = "ome.tiff"
     else:
-        file_writer = args.fw[0]
+        file_writer = args.fw
 
-    reg_graph = config_to_WsiReg2D(config_filepath)
-    reg_graph.add_data_from_config(config_filepath)
-
-    reg_graph.register_images()
-    reg_graph.save_transformations()
-
-    if args.write_im:
-        reg_graph.transform_images(file_writer=file_writer)
-
-    if reg_graph.shape_sets:
-        reg_graph.transform_shapes()
-
-
-if __name__ == "__main__":
-    import sys
-
-    sys.exit(main())
+    main(
+        config_filepath,
+        write_images=args.write_im,
+        to_original_size=args.to_original_size,
+        transform_non_reg=args.transform_non_reg,
+        remove_merged=args.remove_merged,
+        file_writer=file_writer,
+    )
+    sys.exit()
